#pragma once

#include "CoreMinimal.h"
#include "InworldState.h"
#include "Packets.h"
#include "InworldCharacterComponent.h"

#include "InworldPlayerComponent.generated.h"

class USoundWave;
class UAudioCaptureComponent;
class UInworldApiSubsystem;

UCLASS(ClassGroup = (Custom), meta = (BlueprintSpawnableComponent))
class INWORLDAIINTEGRATION_API UInworldPlayerComponent : public UActorComponent, public Inworld::FPacketVisitor, public Inworld::IPlayerComponent
{
	GENERATED_BODY()

public:
    UInworldPlayerComponent();

    virtual void BeginPlay() override;
    virtual void EndPlay(const EEndPlayReason::Type EndPlayReason) override;
    virtual void TickComponent(float DeltaTime, enum ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

    virtual void HandleConnectionStateChanged(EInworldConnectionState State) override;

    virtual void HandlePacket(TSharedPtr<Inworld::FInworldPacket> Packet) override;

	UFUNCTION(BlueprintCallable, Category = "Camera")
    void SetCameraForward(const FVector& InCameraForward);

    virtual Inworld::ICharacterComponent* GetTargetCharacter() override { return TargetCharacter.Get(); }

private:
    virtual void Visit(const Inworld::FTextEvent& Event) override;

    void StartVoiceCapture();
    void StopVoiceCapture();
    void StartVoiceChunkCapture();
    void StopVoiceChunkCapture();

    void UpdateTargetCharacter();
    void SetTargetCharacter(TWeakObjectPtr<UInworldCharacterComponent> ClosestCharacter);
    void ClearTargetCharacter();

    void UpdateVoiceCapture();

private:
    UPROPERTY(EditAnywhere, Category = "UI")
    FString UiName = "Player";

    /** Minimum distance to start interacting with a character */
    UPROPERTY(EditAnywhere, Category = "Interaction")
    float InteractionDistance = 300.f;

    /** How close must player be facing the direction of the character to interact */
    UPROPERTY(EditAnywhere, Category = "Interaction")
	float InteractionDotThreshold = 0.5f;

	const float SoundMessageLength = 0.1f;

    TWeakObjectPtr<UInworldApiSubsystem> InworldSubsystem;
    TWeakObjectPtr<UAudioCaptureComponent> AudioCaptureComponent;

    TWeakObjectPtr<UInworldCharacterComponent> TargetCharacter;
    
    FVector CameraForward;

    bool bCapturingVoice = false;
    float LastSoundMessageTime = 0.f;
};
